.comment Standard DECUS C Documentation layout
.comment
.comment Position the printer to start on the 4th line for 66 line
.comment (11 inch) paper.  The 60 lines printed should be vertically
.comment centered on the page, with 3 line vertical margins top and bottom.
.comment
.comment The title page is laid out so that the title will show through
.comment a "window" cover such as is used on Digital documentation.
.comment
.comment Note: there is a bug in the page number algorithm of RSTS/E
.comment runoff.  Thus, on RSTS/E, GETRNO.C must mangle the page
.comment heading to fix page number alignment.
.comment
.comment The following statement will give a (non-fatal) error on RNO.TSK
.comment It is needed because somebody was clever in designing "Standard
.comment RUNOFF".
.NO AUTOSUBTITLE .STYLE HEADERS 3,0,0
.comment
.nhd.uc.ps 58,80.lm 8.rm 72
#
.s 26
.c ;DECUS C LANGUAGE SYSTEM
.s 2
.c ;Utility Library
.comment wizard
.c ;Software Support Manual
.comment mundane
.s 2
.c ;by
.s
.c ;Jerry Leichter
.s 3
This document describes the RSX/VMS/RSTS/RT11 DECUS C language system
Utility Library.
.comment wizard
It also contains all internal functions.
.comment mundane
.s 2
.c ;DECUS Structured Languages SIG
.s
.c ;Version of 26-Jun-81
.nhy
.p 2, 1, 3
.pg
#
.comment skip a page to put copyright on r.h. page if printed front to back.
.pg
#
.s 12
.note Copyright (C) 1981, DECUS
General permission to copy or modify, but not for profit, is hereby granted,
provided that the above copyright notice is included and reference made to
the fact that reproduction privileges were granted by DECUS.
.s
The information in this document is subject to change without notice
and should not be construed as a commitment by Digital Equipment
Corporation or by DECUS.
.s
Neither Digital Equipment Corporation, DECUS, nor the authors assume
any responsibility for the use or reliability of this document or
the described software.
.s
This software is made available without any support whatsoever.
The person responsible for an implementation of this system
should expect to have to understand and modify the source code if
any problems are encountered in implementing or maintaining the compiler or
its run-time library.  The DECUS 'Structured Languages Special Interest
Group' is the primary focus for communication among users of this software.
.en
.s 10
UNIX is a trademark of Bell Telephone Laboratories.  RSX, RSTS/E, RT11
and VMS are trademarks of Digital Equipment Corporation.
.s
.hd
.hd mixed
.comment The following gives a bug on RNO.TSK
.head mixed
.comment
.ch INTRODUCTION
.ps 58,80.lm 8.rm 72
.hd mixed
.t [[########C Utility Library]]
.comment wizard
.st ########Software Support Manual
.comment mundane
.s
The C Utility Library consists of routines to perform many common functions.
For example, there is a set of routines that handle VSTRINGs, which are
dynamically allocated, variable length strings.  Another set of routines
handle TABLEs, which are hash tables that provide a powerful associative
memory capability.  Some routines replace or extend routines in the standard
library; for example, a more extensive list of "is" functions than is
available in the standard library is contained in the Utility Library, based
on a global table that defines the characteristics of ASCII characters.
.s
The Utility Library also provides an emulation of some obsolete routines,
such as iovtoa(), thus providing compatibility with older versions of the
standard library.
.s
All the routines in the Utility Library are written in C, and every attempt
has been made to make them reasonably clear and easy to modify.  Thus, you
may often be able to use a Utility routine that is close to what you need
as a basis for the exact code you need.
.s
To use the Utility Library, you must do two things:  "include" any required
header files, as documented in the routine descriptions; and arrange for the
Utility Library to be searched when you link or task-build your program.
.s
Generally, the Utility Library header files will be stored in the standard
C account - where stdio.h is.  All the documentation assumes this.  If this
is not the case, you will have to modify your "include" statements
appropriately.
.s
The Utility Library's object modules are stored in CU.OBJ on RT11-based
systems, and in CU.OLB on RSX-based systems.  One of these - or, sometimes,
on RSTS, both - will be stored in the standard C library account - where
C.OLB or CLIB.OBJ is stored.  Your RT11 linker or Task Builder commands
must arrange to search the appropriate account and file.  Note that you
will usually want to search CU BEFORE C or CLIB, so that you get the C Utility
Library versions of routines that exist in both.  Searching C or CLIB first is
possible, but proper functioning cannot be guaranteed, since some of the
Utility Library routines may rely on their ability to call their cohorts
in the Library, rather than the standard library routines of the same name.
Caveat emptor!
.lm 8.rm 72.nhy .head mixed

.st ########blkcmp##Compare two blocks of memory
.pg
.hl 1 ^&Compare two blocks of memory\&
.s 2
.c ;**********
.c ;* blkcmp *
.c ;**********
.s 2
.lm +8.nf
.s.i -8;File name:	blkcmp.c
.s 2
.i -8;Usage
.s
blkcmp(a,b,nbytes)
char *a;
char *b;
unsigned nbytes;
.s
.i -8;Description
.s
.fill
blkcmp compares two blocks of memory, each nbytes long.  Like strcmp,
it returns:
.nf
	-1	a  < b
	 0	a == b
	+1	a  > b
.fill
where the comparison is lexicographical (as for strcmp).
.s
Zero bytes have no special meaning to blkcmp() and are compared
along with everything else.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########blkeq###Test two blocks of memory for equality
.pg
.hl 1 ^&Test two blocks of memory for equality\&
.s 2
.c ;*********
.c ;* blkeq *
.c ;*********
.s 2
.lm +8.nf
.s.i -8;File name:	blkeq.c
.s 2
.i -8;Usage
.s
blkeq(a,b,nbytes)
char *a;
char *b;
unsigned nbytes;
.s
.i -8;Description
.s
.fill
blkeq returns TRUE if a and b point to nbytes bytes of identical
memory.
.s
Zero bytes have no special meaning to blkeq() and are compared
along with everything else.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########chrtab##Header file for the character table chrtab[]
.pg
.hl 1 ^&Header file for the character table chrtab[]\&
.s 2
.c ;**********
.c ;* chrtab *
.c ;**********
.s 2
.lm +8.nf
.s.i -8;File name:	chrtab.h
.s 2
.i -8;Usage
.s
_#include <chrtab.h>
.s
.i -8;Description
.s
.fill
This file defines the compile-time constants used in chrtab[], which
it globalizes.  It is only needed if you want to refer to chrtab[]
directly; the various "is" functions that use the table will work with
or without it.
.s
The characteristics of characters are defined by bits turned on in the
array element corresponding to them.  Thus, chrtab['a'] gives
information about 'a'.  chrtab[] is 128 bytes long and should only be
refered to with 7-bit characters.
.s
At present, no character has more than one bit turned on; this may
change.  Only 5 bits are defined; the first unused bit is FREEBIT.
You can modify the table by OR'ing in bits starting with FREEBIT (up
through bit value 128).
.s
The bits are defined as follows:
.nf
	LOWER		a-z
	UPPER		A-Z
	DIGIT		0-9
	WHITE		<SPACE>, <TAB>, <LF> ('_\n'),
			<VT>, <FF> ('_\f')
	MARK		Any of !.,:;?
.s
.fill
The following useful bit combinations are also defined:
.nf
	ALPHA		LOWER | UPPER
	ALFNUM		ALPHA | DIGIT
.s
.i -8;Bugs
.s
.fill
The definition of whitespace is not the same as the UNIX definition
(which doesn't include <VT> or <FF>).  However, it seems more correct
philosophically.
.s
Perhaps <CR> ('_\r') should also be whitespace, for the same reason.
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########chrtab##Character characteristics table
.pg
.hl 1 ^&Character characteristics table\&
.s 2
.c ;**********
.c ;* chrtab *
.c ;**********
.s 2
.lm +8.nf
.s.i -8;File name:	chrtab.c
.s 2
.i -8;Usage
.s
extern char chrtab[128];
	OR
_#include <chrtab.h>
.s
.i -8;Description
.s
.fill
This file defines a global byte table called chrtab[] that can be used
to determine the characteristics of various characters.  See chrtab.h
for details.
.s
Note that only characters without parity should be used as indices
into chrtab[].
.s
Bugs
.s
The character table is, of course, dependent on the character code
used.  The one defined here is for ASCII characters.
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########esc#####Process escaped characters
.pg
.hl 1 ^&Process escaped characters\&
.s 2
.c ;*******
.c ;* esc *
.c ;*******
.s 2
.lm +8.nf
.s.i -8;File name:	esc.c
.s 2
.i -8;Usage
.s
char esc(ppc)
char **ppc;
.s
extern int esc__msk;
.s
.i -8;Description
.s
.fill
esc(ppc) recognizes and processes the escaped characters recognized by
the C compiler.  If *ppc points to any character other than '_\', esc()
just returns that character.  Otherwise, it examines the next
character.  If that one is one of the set {b,f,n,r,t}, the appropriate
character is returned, e.g., a tab for '_\t'.  Also, _\ddd, ddd being up
to three octal digits, returns the character with value ddd.  Only the
bottom 7 bits are retained, so the result is always a "proper"
character.  This can be changed by changing the global esc__msk to
contain whatever mask you prefer.  (The default is 0177.)
.s
The special characters b, f, and so on are recognized in lower case
only.
.s
If '_\' is the last character in s, '_\' itself is returned.
.s
If '_\' is followed by any other character x, x itself is returned.
.s
In all cases, *ppc is left pointing at the last character that esc()
has "eaten" e.g., at the 't' if it is returning a tab for '_\t'.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########flex####Header file for flexes
.pg
.hl 1 ^&Header file for flexes\&
.s 2
.c ;********
.c ;* flex *
.c ;********
.s 2
.lm +8.nf
.s.i -8;File name:	flex.h
.s 2
.i -8;Usage
.s
_#include <flex.h>
.s
.i -8;Description
.s
.fill
This header file is used to allow programs to access flexes.  See
flex.c for additional details.
.s
This file defines the structure of a flex.  The fields are as follows:
.s
char *fxdata	Pointer  to the  actual data.  This will
.nf
		always be the first  element, so  that a
		flex  can  be  considered to  be of type
		char ** (or  whatever) if you  only want
		to read it.  fxdata will be  NULL if the
		flex has been damaged.
.s
.fill
unsigned fxused	Number of items used in the flex.
.s
unsigned fxdim	Number of items the  flex can hold; 0 if
.nf
		the flex has been damaged.
.s
.fill
unsigned fxisz	Size of one item, in bytes.
.s
unsigned fxext	Number of  items to grow  by;  if 0, the
.nf
		flex won't grow.
.s
.fill
The only fields generally directly accessed by programs are fxdata
and fxlen.  Note that fxdata may change over time as items are added
to and removed from the flex.
.s
The only field you should normally change is fxext, the extension
quantity. The value current the next time the flex must grow will
determine how much it grows by.
.s
flex.h also contains declarations describing all the flex-handling
routines. They are not normally included since they will create
external references to every one of these routines, probably making
your program larger than it has to be.  If you can accept the space
loss, define symbol FXDODEFS before including flex.h.  Alternatively,
you can extract just the declarations you need from this file and
include them in your own code.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########flex####Create a new flex
.pg
.hl 1 ^&Create a new flex\&
.s 2
.c ;********
.c ;* flex *
.c ;********
.s 2
.lm +8.nf
.s.i -8;File name:	flex.c
.s 2
.i -8;Usage
.s
_#include <flex.h>
.s
FLEX *
flex(dim,isz,ext)
unsigned dim;		-- initial size, in items
unsigned isz;		-- size of an item in bytes
unsigned ext;		-- number of items to extend by
.s
.i -8;Description
.s
.fill
Flexes are dynamically expandable arrays.  If an attempt is made to
add an item to a flex that is already full, the flex's core area is
re-allocated to make more room.
.s
All the items in a flex must be the same size; all references are by
item numbers, which, like array indices, run from 0 to (_# items in the
flex minus 1).  When a flex must expand, it expands some whole number
of items; the particular number to expand by is stored in the flex's
header.  If this number is zero, the flex cannot expand.
.s
If a flex cannot expand, either because its header sets a growth size
of 0 or because there is insufficient core, but a request is made to
add an item while it is full, the flex is marked damaged.  The core it
used for its data is lost (realloc() free's it) and any further
attempts to use the flex are ignored.  In general, any time a flex
becomes damaged, or a damaged flex is passed to one of the flex
handling routines, the routine returns NULL.  (Usually, they just
return the flex passed.)
.s
A flex actually consists of a header, which contains information
describing it, and a separate block of data.  The first item in the
header is a pointer to the data.  While the header never moves, the
data may move any time items are added to or removed from the flex.
.s
It is occassionally necessary to recover the information in a damaged
flex (in most cases, there is no reasonable way to continue once the
program runs out of memory).  This can be done by making use of
realloc()'s ability to reallocate the most recently freed block of
core.  Before making the potentially damaging call, save the current
values of the fxdata and fxused elements of the flex.  Should the
flex become damaged, the fxdata value can be passed to realloc()
safely.  Note that this recovers the data, but not the flex's header;
you still cannot apply any of the flex-handling functions to it.
.s
The various flex handling functions in the Utility Library all have
names beginning "fx".  They form a fairly complete set of primitives.
You may wish to restrict yourself to doing all accesses through these
routines since this would, in principle, allow you to replace them
with a set of routines that, for example, page items to disk if a flex
gets too large.
.s
The file flex.h contains some definitions that are useful when dealing
with flexes.
.s
The function flex(dim,isz,ext) creates a new flex.  The data area of
the flex will initially contain sufficient space for dim items, each
isz bytes in length.  If the flex fills, it will be extended in units
of ext items.
.s
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########fxadd###Add an item to a flex
.pg
.hl 1 ^&Add an item to a flex\&
.s 2
.c ;*********
.c ;* fxadd *
.c ;*********
.s 2
.lm +8.nf
.s.i -8;File name:	fxadd.c
.s 2
.i -8;Usage
.s
_#include <flex.h>
.s
FLEX *
fxadd(fx,itm)
register FLEX *fx;
char *itm;
.s
.i -8;Description
.s
.fill
fxadd() adds an item to a flex.  It returns NULL if the flex would
have had to grow but couldn't because of insufficient memory or a zero
extension quantity. In this case, the data in the flex is lost and the
flex itself cannot be used again; further attempts always return NULL.
If all went well, the flex pointer passed is returned.
.s
Note that an item is defined entirely by its length; fxadd() will try
to copy exactly fx->fxisz bytes, regardless of null bytes, etc.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########fxaddo##Add ordered to a flex
.pg
.hl 1 ^&Add ordered to a flex\&
.s 2
.c ;**********
.c ;* fxaddo *
.c ;**********
.s 2
.lm +8.nf
.s.i -8;File name:	fxaddo.c
.s 2
.i -8;Usage
.s
_#include <flex.h>
.s
FLEX *fxaddo(fx,itm,ord)
FLEX *fx;
char *itm;
int (*ord)();
.s
unsigned fxladd;
.s
.i -8;Description
.s
.fill
fxaddo is similar to fxadd except that it ensures that the items in
the flex are in increasing order, where "in order" is defined by a
passed function, ord.  ord must return values akin to strcmp:
ord(a,b) is -1 if a<b, 0 if a=b, and 1 if a>b.  Elements that compare
equal will be stored in the order they are passed.
.s
The global fxladd gets the item number at which itm was added.
.s
W A R N I N G !
.s
fxaddo() uses straight insertion to maintain the ordering.  This is an
O(n**2) algorithm and will become very slow when the flex gets large.
If you need to build a large, sorted flex, it is probably best to fill
it first and then use something like qqsort() to sort it when you are
done.  (Another approach, more in keeping with the philosophy of
fxaddo(), would be to define, say, fxaddh(), which keeps the flex
ordered as a heap, and then scan through it as a heap.  I leave this
to someone else.)
.s
Note:  If for some reason you don't want to include equal elements,
you can safely issue an error message in ord() and use, for example,
setexit() and unwind() to abort the insertion.  This process will
leave the flex unchanged.  Alternatively, set a flag and use fxladd to
eject the item later.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########fxeject#Eject (delete) an item from a flex
.pg
.hl 1 ^&Eject (delete) an item from a flex\&
.s 2
.c ;***********
.c ;* fxeject *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	fxejec.c
.s 2
.i -8;Usage
.s
_#include <flex.h>
.s
FLEX *
fxeject(fx,n)
FLEX *fx;
unsigned n;
.s
.i -8;Description
.s
.fill
fxeject(fx,n) removes item n from the flex fx, moving all later items
in to fill in the gap.  The space allocated for the flex is not
changed, however.
.s
fxeject() returns fx if all went well, NULL if n was not a legal item
number or fx was damaged.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########fxfree##Free a flex
.pg
.hl 1 ^&Free a flex\&
.s 2
.c ;**********
.c ;* fxfree *
.c ;**********
.s 2
.lm +8.nf
.s.i -8;File name:	fxfree.c
.s 2
.i -8;Usage
.s
_#include <flex.h>
.s
fxfree(fx)
FLEX *fx;
.s
.i -8;Description
.s
.fill
Free a flex.  Attempting to free something that was not created with
flex() will cause trouble.  However, NULL or a flex that was damaged
may be passed safely.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########fxinjec#t	Inject an item into a flex
.pg
.hl 1 ^&t	Inject an item into a flex\&
.s 2
.c ;***********
.c ;* fxinjec *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	fxinje.c
.s 2
.i -8;Usage
.s
_#include <flex.h>
.s
FLEX *
fxinject(fx,itm,n)
FLEX *fx;
char *itm;
unsigned n;
.s
.i -8;Description
.s
.fill
fxinject(fx,itm,n) injects itm into flex fx so that it becomes item
number n; items already in the flex are moved to make room.  Another
way to look at this is that fxinject injects itm before item n in fx,
and allows the special case of n=1+(greatest item number) for
injection at the end (which is functionally identical to fxadd()).
.s
fxinject() returns fx if all went well, otherwise NULL.  fx is never
expanded or marked damaged if n is out of range.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########fxitem##Obtain (the address of) an item from a flex
.pg
.hl 1 ^&Obtain (the address of) an item from a flex\&
.s 2
.c ;**********
.c ;* fxitem *
.c ;**********
.s 2
.lm +8.nf
.s.i -8;File name:	fxitem.c
.s 2
.i -8;Usage
.s
_#include <flex.h>
.s
char *
fxitem(fx,n)
register FLEX *fx;
unsigned n;
.s
.i -8;Description
.s
.fill
fxitem(fx,n) returns a pointer to the n'th item in the flex, if there
is one, or NULL if there isn't or the flex has been damaged.  The
first item in the flex has item number 0.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########fxneed##Explicitly grow or shrink a flex
.pg
.hl 1 ^&Explicitly grow or shrink a flex\&
.s 2
.c ;**********
.c ;* fxneed *
.c ;**********
.s 2
.lm +8.nf
.s.i -8;File name:	fxneed.c
.s 2
.i -8;Usage
.s
_#include <flex.h>
.s
FLEX *fxneed(fx,cnt)
FLEX *fx;
unsigned cnt;
.s
.i -8;Description
.s
.fill
Ensure that a flex contains room for some entries.  fxneed(fx,cnt)
expands the flex, if possible, so that at least cnt items will fit
with no further expansion.  (If fx is expanded, it is expanded so
that it will have room for EXACTLY cnt more items.)
.s
If cnt=0, the flex is reduced in size, if necessary, so that it has
no empty space in it.  (The extension quantity is not altered; you
can add to the flex again later.)
.s
fxneed() ignores the extension quantity (fxext) specified for the
flex; it will extend exactly as much as it needs to, and will extend
even when fxext is 0.
.s
If all goes well, fx is returned; otherwise, NULL is returned and
the flex is marked damaged.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########fxset###Set the value of an item in a flex
.pg
.hl 1 ^&Set the value of an item in a flex\&
.s 2
.c ;*********
.c ;* fxset *
.c ;*********
.s 2
.lm +8.nf
.s.i -8;File name:	fxset.c
.s 2
.i -8;Usage
.s
_#include <flex.h>
.s
FLEX *
fxset(fx,itm,n)
FLEX *fx;
char *itm;
unsigned n;
.s
.i -8;Description
.s
.fill
fxset(fx,itm,n) changes the value of item n in fx to itm.  It is
mainly useful for "information-hiding"; if you always use fxset() to
set elements, rather than using the address returned by fxitem()
directly, you'll find it much easier to use a different set of flex
handlers - e.g. flex handlers that page data to disk and need to know
when in-core data has been changed.  Also, it could easily produce
some sort of immediate "index out of range" error, instead of just
returning NULL (for easier debugging).
.s
fxset() returns fx itself if all went well, and NULL if fx was damaged
or had no item n.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########index###Get index of character in string
.pg
.hl 1 ^&Get index of character in string\&
.s 2
.c ;*********
.c ;* index *
.c ;*********
.s 2
.lm +8.nf
.s.i -8;File name:	index.c
.s 2
.i -8;Usage
.s
index(s,c)
char s[];
char c;
.s
.i -8;Description
.s
.fill
The function index(s,c) returns the index in the string pointed to by
s of the character c - i.e., s[index(s,c)] is the first c in s.  If c
does not occur in s, index() returns -1.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########instr###Get index of a string in a string
.pg
.hl 1 ^&Get index of a string in a string\&
.s 2
.c ;*********
.c ;* instr *
.c ;*********
.s 2
.lm +8.nf
.s.i -8;File name:	instr.c
.s 2
.i -8;Usage
.s
instr(s,p)
char s[];			-- Subject
char p[];			-- Pattern
.s
.i -8;Description
.s
.fill
instr(s,p) returns the index of the first occurence of *p in *s, or
-1 if p is not a substring of s.  (The index is the offset to the
beginning of the occurence; for example, instr("abcd","bc")==1.)
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########iovtoa##Convert file name to Ascii - obsolete
.pg
.hl 1 ^&Convert file name to Ascii - obsolete\&
.s 2
.c ;**********
.c ;* iovtoa *
.c ;**********
.s 2
.lm +8.nf
.s.i -8;File name:	iovtoa.c
.s 2
.i -8;Usage
.s
char *
iovtoa(fd,buffer);
FILE *fd;
char *buffer;
.s
.i -8;Description
.s
.fill
iovtoa() is an obsolete name and form of fgetname().  It exists
solely for compatibility purposes; new programs should use fgetname()
and old ones SHOULD (eventually) be converted.
.s
Note that iovtoa() differs from fgetname() in only one way:  While
fgetname() returns buffer, iovtoa() returns a pointer to the null
trailing the extracted name.
.s
.i -8;Bugs
.s
Obsolete.
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########isalfnu#m	Test if a character is alphanumeric
.pg
.hl 1 ^&m	Test if a character is alphanumeric\&
.s 2
.c ;***********
.c ;* isalfnu *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	isalfn.c
.s 2
.i -8;Usage
.s
isalfnum(c)
char c;
.s
.i -8;Description
.s
.fill
isalfnum(c) returns 1 if c is letter or a digit; otherwise, it returns
0.
.s
Note the spelling of the name!
.s
.i -8;Bugs
.s
VMS documentation calls '$' and '__' alphanumeric; isalfnum() disagrees.
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########isalpha#Test if a character is alphabetic
.pg
.hl 1 ^&Test if a character is alphabetic\&
.s 2
.c ;***********
.c ;* isalpha *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	isalph.c
.s 2
.i -8;Usage
.s
isalpha(c)
char c;
.s
.i -8;Description
.s
.fill
isalpha(c) returns 1 if c is an upper or lower case letter; otherwise,
it returns 0.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########isdigit#Test if a character is a digit
.pg
.hl 1 ^&Test if a character is a digit\&
.s 2
.c ;***********
.c ;* isdigit *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	isdigi.c
.s 2
.i -8;Usage
.s
isdigit(c)
char c;
.s
.i -8;Description
.s
.fill
isdigit(c) returns 1 if c is a digit; otherwise, it returns 0.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########islower#Test if a character is lowercase
.pg
.hl 1 ^&Test if a character is lowercase\&
.s 2
.c ;***********
.c ;* islower *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	islowe.c
.s 2
.i -8;Usage
.s
islower(c)
char c;
.s
.i -8;Description
.s
.fill
islower(c) returns 1 if c is a lower case letter; otherwise, it
returns 0.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########isupper#Test if a character is uppercase
.pg
.hl 1 ^&Test if a character is uppercase\&
.s 2
.c ;***********
.c ;* isupper *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	isuppe.c
.s 2
.i -8;Usage
.s
isupper(c)
char c;
.s
.i -8;Description
.s
.fill
isupper(c) returns 1 if c is an upper case letter; otherwise, it
returns 0.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########iswhite#Test if a character is whitespace
.pg
.hl 1 ^&Test if a character is whitespace\&
.s 2
.c ;***********
.c ;* iswhite *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	iswhit.c
.s 2
.i -8;Usage
.s
iswhite(c)
char c;
.s
.i -8;Description
.s
.fill
iswhite(c) returns 1 if c is a whitespace; otherwise, it returns 0.
.s
"Whitespaceness" is defined by chrtab[]; see chrtab.h.  The following
are whitespace:  <SPACE>, <TAB>, <LF> ('_\n'), <VT>, and <FF> ('_\f').
.s
.i -8;Bugs
.s
This is not the same as the UNIX definition (which doesn't include
<VT> or <FF>).  However, it seems more correct philosophically.
.s
Perhaps <CR> ('_\r') should also be whitespace, for the same reason.
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########match###Determine if a string is a leading substring of another
.pg
.hl 1 ^&Determine if a string is a leading substring of another\&
.s 2
.c ;*********
.c ;* match *
.c ;*********
.s 2
.lm +8.nf
.s.i -8;File name:	match.c
.s 2
.i -8;Usage
.s
char *
match(s,p)
char *s;
char *p;
.s
.i -8;Description
.s
.fill
match(s,p) returns a pointer to the first character of s beyond those
matching p, if p is an initial substring of s.  Otherwise, it returns
NULL.  Thus, match("abcde","abc") returns a pointer to the 'd' in the
first string; match("abcde","bc") return NULL.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########qsort###Fast in-core sort
.pg
.hl 1 ^&Fast in-core sort\&
.s 2
.c ;*********
.c ;* qsort *
.c ;*********
.s 2
.lm +8.nf
.s.i -8;File name:	qsort.c
.s 2
.i -8;Usage
.s
qsort(array,nelements,element__size,compare)
char		*array;		-- Sort this array
unsigned	nelements;	-- How many elements
unsigned	element__size;	-- Size in bytes of an element
int		(*compare)();	-- Called to compare stuff
.s
.i -8;Description
.s
.fill
qsort() is a general-purpose in-core sort function based on the
quicksort algorithm.  It is passed an array of elements to be sorted,
the number of elements in the array, the size in bytes of each of
those elements, and a function that compares elements.  compare() must
return an int such that compare(a,b) is <, =, or > 0 when a is to be
considered <, =, or > b in the comparison.  Compare will receive two
arguments - pointers to items a and b (array[] elements).  Sorting is
in increasing order - i.e., when qsort() returns, array[i] will
compare() less than or equal to array[j] whenever i < j.
.s
Note that compare() is the ONLY function that qsort() ever uses to
compare elements; it can thus return 0 for objectively different
elements - for example, it can consider upper- and lower-case letters
to be the same.
.s
.i -8;Internal
.s
qsort() is based on the Hoare quicksort algorithm.  It was originally
translated from a RATFOR version in "Software Tools" by Martin Minow.
Jerry Leichter modified it to conform with various recommendations
found in the article "Implementing Quicksort Programs" by Robert
Sedgewick (CACM Vol. 21 _#10 (Oct. 1978) pg. 847).  This should speed
up the algorithm somewhat in all cases, and will result in a dramatic
improvement if qsort() is passed an already-sorted (or almost sorted)
array.  (The original algorithm then displays O(n_^2) behaviour; the
improved one is, if anything, faster than on random files.)
.s
See the source for sortc (in the tools library) for yet another
implementation of qsort.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########replace#Simple character translation
.pg
.hl 1 ^&Simple character translation\&
.s 2
.c ;***********
.c ;* replace *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	replac.c
.s 2
.i -8;Usage
.s
extern char rep__tab[256];
.s
char *
replace(s,from,to)
char s[];
char from[];
char to[];
.s
.i -8;Description
.s
.fill
replace() provides a simple interface to translate() and the functions
associated with it; see transl.c for details.  Briefly, each character
character in s that occurs in from is replaced by the corresponding
character in to.  If there are more characters in from than in to,
the "excess" characters are deleted from the output string.  replace()
returns the output string in space allocated from malloc(); the input
string is unchanged.  If replace() can't get the space it needs, it
returns NULL.
.s
rep__tab is a global character table that is used in the internal calls
to the translation routines.  It is large enough to translate any
8-bit byte.  rep__tab is cleared and the set up on each call to
replace().  If you intend to call replace() many times with the same
second and third arguments, you can call it once to set up rep__tab and
then call translate() directly - change subsequent s1 = replace(s,t,f)
calls to translate(rep__tab,s1 = savestr(s)).
.s
Note: The power of a function like replace() is often not appreciated.
If s is any 5-character string, then replace("abcde","edcba",s) is s
reversed - i.e., this call maps "words" to "sdrow".  In general, if
P is any permutation of n-character strings, n<=256, then the call
replace(b,P(b),s) returns P(s), where b is any (constant) string of
length n containing n different characters.  (s, of course, is also of
length n).  Thus, replace() implements character substitutions,
deletions, and permutations, all in one small, simple to use function.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########savestr#Make a safe copy of a string
.pg
.hl 1 ^&Make a safe copy of a string\&
.s 2
.c ;***********
.c ;* savestr *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	savest.c
.s 2
.i -8;Usage
.s
char *savestr(s)
char s[];
.s
.i -8;Description
.s
.fill
savestr(s) returns a pointer to a newly-allocated block of memory
containing a copy of s.  The memory is obtained from malloc() and
should be freed with free() when you are done with it.
.s
savestr() returns NULL if it cannot obtain the memory it needs.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########strfns##Simple string functions
.pg
.hl 1 ^&Simple string functions\&
.s 2
.c ;**********
.c ;* strfns *
.c ;**********
.s 2
.lm +8.nf
.s.i -8;File name:	strfns.c
.s 2
.i -8;Usage
.s
char *ctos(c)
char c;
.s
char *scopy(s)
char s[];
.s
char *scat(s1,s2)
char s1[],s2[];
.s
char *mid(s,n,l)
char s[];
int n,l;
.s
char *seg(s,n,l)
char s[];
int n,l;
.s
char *right(s,l)
char s[];
int l;
.s
char *left(s,n)
char s[];
int n;
.s
.i -8;Description
.s
.fill
The functions defined here provide simple manipulation of character
strings a la BASIC PLUS.  The strings returned are all new - i.e.
not substrings of the strings passed.  All returned strings are in
space acquired from talloc() (q.v.); you should set a mark before
calling these functions and then used tfree() to clean up.
.s
A return of NULL indicates a fatal error - usually it means that the
function couldn't allocate space for the string it wanted to make.
.s
ctos(c)		returns a new string  containing  single
.nf
		character c.
.s
.fill
scopy(s)	returns a fresh copy of s.
.s
scat(s,t)	concatenates s and t.
.s
mid(s,n,l)	extracts l characters from s starting at
.nf
		character  n (first  character  is  n=0,
		etc.).  If the substring specified  does
		not exist, the part that does is extrac-
		ted.  For example,  n<0 is equivalent to
		n=0.
.s
.fill
seg(s,n,m)	is the substring of  characters in posi-
.nf
		tions n through m inclusive.  (The first
		character is position  0).  If n>m, they
		are  exchanged.   As with mid(), if  the
		"substring"  specified overflows s, only
		the "proper" part is extracted.
.s
.fill
left(s,l)	extracts the leftmost l characters of s,
.nf
		with "overflow" handled as for mid().
.s
.fill
right(s,n)	is the substring of characters  starting
.nf
		at character n, where the  first charac-
		ter is character 0; overflow handling is
		as for mid().
.s
.fill
Note:  These functions have been written to be simple to use, not to
be particularly efficient.  If you are going to be doing large amounts
of string processing, you will almost certainly want to use different
techniques.  Don't forget that every string created in space allocated
by talloc() requires 4 words of overhead - 2 for malloc() and 1 for
talloc().
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########tables##Hash table handler functions
.pg
.hl 1 ^&Hash table handler functions\&
.s 2
.c ;**********
.c ;* tables *
.c ;**********
.s 2
.lm +8.nf
.s.i -8;File name:	tables.c
.s 2
.i -8;Usage
.s
typedef char TABLE;
.s
TABLE *
table(size)
int size;
.s
tdelete(t,key)
TABLE *t;
char *key;
.s
tinsert(t,key,value)
TABLE *t;
char *key;
int value;
.s
tlookup(t,key)
TABLE *t;
char *key;
.s
tnext(t,n,key,value)
TABLE *t;
int n;
char **key;
int *value;
.s
tupdate(t,key,value)
TABLE *t;
char *key;
int value;
.s
.i -8;##Internal
.s
The actual datatypes used are:
.s
typedef struct
{	char *key;
	int value;
} PAIR;
.s
typedef struct
{	int tsize;
	PAIR tdata[];
} TABLE;
.s
PAIR *
$$locate(t,key)
TABLE *t;
char *key;
.s
.i -8;Description
.s
.fill
Tables are associative memory.  They consist of keys and associated
values.  You can add, delete, and change pairs of keys and values and
look up the value associated with a key.
.s
Keys are arbitrary strings; values are arbitrary integers, except that
NULL should not be used as a value since tlookup() returns NULL to
indicate that a key was not found in the table.  In many cases,
value is actually a pointer to a string.
.s
Note the difference in the handling of key and value.  The functions
automatically copy the string that key points to (with savestr()) when
they create a new table entry; hence, you can pass a pointer to a
string in a buffer or anywhere.  value, however, is treated as a
simple int; if it points to a string somewhere, it is your
responsibility to make sure that that string remains unchanged as long
as you need it.
.s
Tables can be freed with free(); however, note that this will not
free the space used for the keys in the table.  You must free this
space, and space used for value strings, if any; use tnext() to
find all the elements.
.s
table(size) creates a new table and returns a pointer to it.  The
table has room for size elements, and cannot grow; however, the space
used for deleted elements is reclaimed automatically.  table() returns
NULL if it cannot obtain (from malloc()) the space it needs -
2*sizeof(char *)*size bytes, plus a small fixed overhead.
.s
Tables are implemented using a hashing technique.  Accesses will be
fast as long as the table does not become too full; however,
performance will degrade rapidly if the table fills.  A good rule of
thumb is to allow space for 10_% more elements than you will actually
need to store.
.s
tdelete(t,key) deletes key and its associated value from t.  If all
goes well, 0 is returned; if key was not in t, 1 is returned and t
is unaltered.  Note that only the space that tinsert() allocated -
for the key - is freed automatically; you must free any space used
by the value associated with key.
.s
tinsert(t,key,value) inserts value as the value associated with
key in table t.  If all went well, tinsert() returns 0.  If the table
is full, or no space could be allocated to copy the key, tinsert()
returns -1.  If key is already in t, tinsert() returns 1; the value
currently associated with key is not changed.
.s
tlookup(t,key) returns the value associated with key in t or NULL if
key isn't found.
.s
tnext() is used to scan through all elements in a table in sequence.
It is used thus:
.s
.nf
	char *key;
	int value;
	for (n = -1; (n = tnext(t,n,_&key,_&value)) >= 0;)
		{ process pair (key, value) }
.s
.fill
Unpredictable results will occur if you insert new elements into a
table (with tinsert()) while scanning it.  The other table functions
may be called with no problems.
.s
CAUTION:  The string pointed to by key as returned is the only copy
available in the table.  Treat it as read-only!
.s
tupdate(t,key,value) changes the value associated with key to value.
tupdate() returns 0 if all went well, 1 if key was not found in t.
In the latter case, t is unaltered.
.s
.i -8;Internal
.s
$$locate(t,key) locates key in table t.  It returns the address of an
entry in the table.  That entry may be EMPTY or DELETED, in which case
the caller may store key into it.  If NULL is returned, the entry
wasn't found and no free entry could be found either.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########talloc##Threaded storage allocator functions
.pg
.hl 1 ^&Threaded storage allocator functions\&
.s 2
.c ;**********
.c ;* talloc *
.c ;**********
.s 2
.lm +8.nf
.s.i -8;File name:	talloc.c
.s 2
.i -8;Usage
.s
char *
talloc(n)
unsigned n;
.s
tfree(p)
char *p;
.s
extern char *tmark;
.s
.i -8;Description
.s
.fill
talloc() is like malloc(); it allocates space.  However, all this
space is linked (threaded) together, and can be freed easily.  Thus:
.s
.nf
	here = tmark;
	a = talloc(5);
	b = talloc(5);
	...
	tfree(here);
.s
.fill
The call to tfree() deletes a, b, and anything else allocated by
talloc() between the point at which tmark was sampled and the call to
tfree().
.s
Like malloc(), talloc() returns NULL if it can't get you space.
.s
It is safe to save tmark more than once; for example, if a function
knows its caller has saved tmark, it needn't ensure that it always
frees its space after an error; the caller's tfree() will get rid of
everything.
.s
talloc() calls malloc(), and malloc() can be used without interfering
with talloc(); tfree() has no effect on space allocated directly with
malloc().
.s
Space allocated with talloc() must NEVER be passed to realloc() or
free(); the pointers differ.
.s
Needless to say, passing a value to tfree() that was not obtained from
tmark or which is obsolete because a value obtained earlier was
already passed will cause trouble.
.s
The overhead involved in using talloc() instead of malloc() is one
(char *) per call.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########time####Define time buffer structure for ctime() and rtime()
.pg
.hl 1 ^&Define time buffer structure for ctime() and rtime()\&
.s 2
.c ;********
.c ;* time *
.c ;********
.s 2
.lm +8.nf
.s.i -8;File name:	time.h
.s 2
.i -8;Usage
.s
_#include <time.h>
   ...
struct TIME buffer;
.s
.i -8;Description
.s
.fill
This header file defines the structure of the time buffer used by
the standard library functions ctime() and rtime().  It is defined
as follows:
.s
struct TIME {
.nf
	int	year;		G.TIYR  year - 1900
	int	month;		G.TIMO  Jan. = 1
	int	day;		G.TIDA
	int	hour;		G.TIHR  00 .. 23
	int	minute;		G.TIMI  00 .. 59
	int	second;		G.TISC  00 .. 59
	int	tick;		G.TICT  00 .. tsec-1
	int	tsec;		G.TICP  tick/second
.fill
};
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########transla#te	Character translation functions
.pg
.hl 1 ^&te	Character translation functions\&
.s 2
.c ;***********
.c ;* transla *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	transl.c
.s 2
.i -8;Usage
.s
clrttab(t,max)
char t[];
int max;
.s
setttab(t,from,to)
char t[];
char *from;
char *to;
.s
translate(t,s)
char t[];
char *s;
.s
.i -8;Description
.s
.fill
These functions provide a character translation facility.  tranlate()
does the actual translation; clrttab() and settab() set up a table
for use with translate().
.s
A translation table t is simple a byte array; translate() will map
character c into t[c].  However, if t[c] is 0, it does not appear
in the output - i.e., translating a character to null deletes it.
t can be from 1 to 256 bytes in length.  Only clrttab(), however,
accepts the length of t as an argument; the other functions assume
that any character they try to translate can be safely looked up in
t.  Usually, t is 128 bytes long and only legal ASCII characters,
without parity, are passed.
.s
clrttab(t,max) clears a table to the null translation - i.e., t will
translate every character into itself.  t is assumed to be max+1 bytes
long (so that it can translate characters with values from 0 to max).
.s
setttab(t,from,to) modifies t so that characters in from are mapped
into corresponding characters in to.  If from is longer than to, any
"excess" characters will map to 0, i.e., be deleted.  Thus, the calls
.s
.nf
	clrttab(t,127);
	setttab(t,"abcdewxyz","ABCDE");
.s
.fill
create a table that maps a,b,c,d and e into their upper-case
equivalents and deletes w,x,y and z.
.s
Note:  It is meaningless to set up a translation for the 0 byte;
t[0] is never used.
.s
translate(t,s) translates string s using table t.  s itself is
modified.  s will be properly null-trailed, but may be shorter than
it was as passed (if any of its characters are deleted).
.s
See also replace() for a simple way to use these functions.
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########trim####Trim trailing whitespace from a string
.pg
.hl 1 ^&Trim trailing whitespace from a string\&
.s 2
.c ;********
.c ;* trim *
.c ;********
.s 2
.lm +8.nf
.s.i -8;File name:	trim.c
.s 2
.i -8;Usage
.s
trim(s)
char s[];
.s
.i -8;Description
.s
.fill
trim() trims all trailing whitespace from the string passed.  Note
that the actual string passed is modified.
.s
"Whitespace-ness" is determined by the function iswhite().
.s
.i -8;Bugs
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########vstring#Header file for vstrings
.pg
.hl 1 ^&Header file for vstrings\&
.s 2
.c ;***********
.c ;* vstring *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	vstrin.h
.s 2
.i -8;Usage
.s
_#include <vstrin.h>
.s
.i -8;Description
.s
.fill
This header file is used to allow programs to access vstrings.  See
vstring.c for additional details.
.s
This file defines the structure of a vstring. The fields are:
.s
char *vsdata	Pointer  to  the actual data.  This will
.nf
		always be  the  first  field,  so that a
		vstring  can be considered to be of type
		char **  if  you  only want  to read it.
		vsdata  will  be NULL if the vstring has
		been damaged.
.s
.fill
unsigned vslen	Number  of bytes  stored in the vstring.
.s
unsigned vsdim	Total number of bytes there is room for.
.s
unsigned vsext	Allocation  quantum  -  the  amount  the
.nf
		vstring is to grow by if it fills. If 0,
		the vstring cannot grow.
.s
.fill
The only fields generally directly accessed by programs are vsdata and
vslen. Note that vsdata may change over time as items are added to and
removed from the vstring.
.s
The only field you should normally change is vsext, the extension
quantum.  The value current the next time the vstring must grow will
determine how much it grows by.
.s
vstring.h also includes external declarations of the appropriate type
for the functions in vstring.c.
.s
.i -8;Bugs
.s
A vstring is really just a special case of a flex with item size 1.
It has been retained as a separate datatype because the type of
operations done on strings are not always generalizable.  For example,
there is no obvious use for an fxadds().
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.st ########vstring#Functions to manipulate vstrings
.pg
.hl 1 ^&Functions to manipulate vstrings\&
.s 2
.c ;***********
.c ;* vstring *
.c ;***********
.s 2
.lm +8.nf
.s.i -8;File name:	vstrin.c
.s 2
.i -8;Usage
.s
_#include <vstrin.h>
.s
VSTRING *
vstring(dim,ext)
unsigned dim;
unsigned ext;
.s
VSTRING *
vsaddc(pv,c)
VSTRING *pv;
char c;
.s
VSTRING *
vsadds(pv,s)
VSTRING *pv;
char *s;
.s
vsfree(pv)
VSTRING *pv;
.s
.i -8;Description
.s
.fill
Vstrings are dynamically expandable strings.  They have room for some
fixed number of characters, but can grow, obtaining memory through
malloc(), if necessary.
.s
vstring(dim,ext) returns a pointer to a new vstring, or NULL if there
is no space for one.  The vstring initially has room for dim bytes,
and will grow by ext bytes if it fills.
.s
vsaddc() adds a character to a vstring.
vsadds() adds a string to a vstring.
.s
vsadds() merely calls vsaddc() in a loop. The trailing EOS is NOT
added.
.s
Both vsaddc() and vsadds() return NULL if the vstring would have had
to grow but couldn't due to lack of memory or a zero growth quantum.
In this case, the data in the vstring is lost and the vstring itself
is marked damaged and cannot be used again; further attempts always
return NULL.
.s
If all went well, the vstring's address as passed is returned.
.s
It is occassionally necessary to recover the information in a damaged
vstring (in most cases, there is no reasonable way to continue once
the program runs out of memory).  This can be done by making use of
realloc()'s ability to reallocate the most recently freed block of
core.  Before making the potentially damaging call, save the current
values of the vsdata and vsused elements of the vstring.  Should the
vstring become damaged, the vsdata value can be passed to realloc()
safely.  Note that this recovers the data, but not the vstring's
header; you still cannot apply any of the vstring-handling functions
to it.
.s
vsfree() frees a vstring.  Attempting to free something that was not
created with vstring() will cause trouble.  However, NULL or a vstring
that was filled and marked damaged may be safely passed.
.s
.i -8;Bugs
.s
A vstring is a special case of a flex which always has an item size
of 1 byte.  It is convenient to maintain it as a separate data type
because the kinds of operations done on the two differ.  For example,
fxadd() takes the address of the item to add; vsaddc() takes the
character itself, which is often more convenient.  The basic set of
functions provided for vstrings is more limited; perhaps more purely
string-oriented functions that work on vstrings will be defined later.
.s
.lm -8.fill
.lm 8.rm 72.nhy .head mixed

.ps 58,80.lm 8.rm 72
.ax USEAGE INFORMATION
.t [[########C Utility Library]]
.st ########Useage Information

.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;blkcmp	Compare two blocks of memory
.s 2
.s
blkcmp(a,b,nbytes)
char *a;
char *b;
unsigned nbytes;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;blkeq	Test two blocks of memory for equality
.s 2
.s
blkeq(a,b,nbytes)
char *a;
char *b;
unsigned nbytes;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;chrtab	Header file for the character table chrtab[]
.s 2
.s
_#include <chrtab.h>
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;chrtab	Character characteristics table
.s 2
.s
extern char chrtab[128];
	OR
_#include <chrtab.h>
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;esc	Process escaped characters
.s 2
.s
char esc(ppc)
char **ppc;
.s
extern int esc__msk;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;flex	Header file for flexes
.s 2
.s
_#include <flex.h>
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;flex	Create a new flex
.s 2
.s
_#include <flex.h>
.s
FLEX *
flex(dim,isz,ext)
unsigned dim;		-- initial size, in items
unsigned isz;		-- size of an item in bytes
unsigned ext;		-- number of items to extend by
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;fxadd	Add an item to a flex
.s 2
.s
_#include <flex.h>
.s
FLEX *
fxadd(fx,itm)
register FLEX *fx;
char *itm;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;fxaddo	Add ordered to a flex
.s 2
.s
_#include <flex.h>
.s
FLEX *fxaddo(fx,itm,ord)
FLEX *fx;
char *itm;
int (*ord)();
.s
unsigned fxladd;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;fxeject	Eject (delete) an item from a flex
.s 2
.s
_#include <flex.h>
.s
FLEX *
fxeject(fx,n)
FLEX *fx;
unsigned n;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;fxfree	Free a flex
.s 2
.s
_#include <flex.h>
.s
fxfree(fx)
FLEX *fx;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;fxinjec	t	Inject an item into a flex
.s 2
.s
_#include <flex.h>
.s
FLEX *
fxinject(fx,itm,n)
FLEX *fx;
char *itm;
unsigned n;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;fxitem	Obtain (the address of) an item from a flex
.s 2
.s
_#include <flex.h>
.s
char *
fxitem(fx,n)
register FLEX *fx;
unsigned n;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;fxneed	Explicitly grow or shrink a flex
.s 2
.s
_#include <flex.h>
.s
FLEX *fxneed(fx,cnt)
FLEX *fx;
unsigned cnt;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;fxset	Set the value of an item in a flex
.s 2
.s
_#include <flex.h>
.s
FLEX *
fxset(fx,itm,n)
FLEX *fx;
char *itm;
unsigned n;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;index	Get index of character in string
.s 2
.s
index(s,c)
char s[];
char c;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;instr	Get index of a string in a string
.s 2
.s
instr(s,p)
char s[];			-- Subject
char p[];			-- Pattern
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;iovtoa	Convert file name to Ascii - obsolete
.s 2
.s
char *
iovtoa(fd,buffer);
FILE *fd;
char *buffer;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;isalfnu	m	Test if a character is alphanumeric
.s 2
.s
isalfnum(c)
char c;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;isalpha	Test if a character is alphabetic
.s 2
.s
isalpha(c)
char c;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;isdigit	Test if a character is a digit
.s 2
.s
isdigit(c)
char c;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;islower	Test if a character is lowercase
.s 2
.s
islower(c)
char c;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;isupper	Test if a character is uppercase
.s 2
.s
isupper(c)
char c;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;iswhite	Test if a character is whitespace
.s 2
.s
iswhite(c)
char c;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;match	Determine if a string is a leading substring of another
.s 2
.s
char *
match(s,p)
char *s;
char *p;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;qsort	Fast in-core sort
.s 2
.s
qsort(array,nelements,element__size,compare)
char		*array;		-- Sort this array
unsigned	nelements;	-- How many elements
unsigned	element__size;	-- Size in bytes of an element
int		(*compare)();	-- Called to compare stuff
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;replace	Simple character translation
.s 2
.s
extern char rep__tab[256];
.s
char *
replace(s,from,to)
char s[];
char from[];
char to[];
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;savestr	Make a safe copy of a string
.s 2
.s
char *savestr(s)
char s[];
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;strfns	Simple string functions
.s 2
.s
char *ctos(c)
char c;
.s
char *scopy(s)
char s[];
.s
char *scat(s1,s2)
char s1[],s2[];
.s
char *mid(s,n,l)
char s[];
int n,l;
.s
char *seg(s,n,l)
char s[];
int n,l;
.s
char *right(s,l)
char s[];
int l;
.s
char *left(s,n)
char s[];
int n;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;tables	Hash table handler functions
.s 2
.s
typedef char TABLE;
.s
TABLE *
table(size)
int size;
.s
tdelete(t,key)
TABLE *t;
char *key;
.s
tinsert(t,key,value)
TABLE *t;
char *key;
int value;
.s
tlookup(t,key)
TABLE *t;
char *key;
.s
tnext(t,n,key,value)
TABLE *t;
int n;
char **key;
int *value;
.s
tupdate(t,key,value)
TABLE *t;
char *key;
int value;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;talloc	Threaded storage allocator functions
.s 2
.s
char *
talloc(n)
unsigned n;
.s
tfree(p)
char *p;
.s
extern char *tmark;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;time	Define time buffer structure for ctime() and rtime()
.s 2
.s
_#include <time.h>
   ...
struct TIME buffer;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;transla	te	Character translation functions
.s 2
.s
clrttab(t,max)
char t[];
int max;
.s
setttab(t,from,to)
char t[];
char *from;
char *to;
.s
translate(t,s)
char t[];
char *s;
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;trim	Trim trailing whitespace from a string
.s 2
.s
trim(s)
char s[];
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;vstring	Header file for vstrings
.s 2
.s
_#include <vstrin.h>
.s
.lm 8.rm 72.nhy .head mixed

.tp 10.s 4.lm +8.nf
.i -8 ;vstring	Functions to manipulate vstrings
.s 2
.s
_#include <vstrin.h>
.s
VSTRING *
vstring(dim,ext)
unsigned dim;
unsigned ext;
.s
VSTRING *
vsaddc(pv,c)
VSTRING *pv;
char c;
.s
VSTRING *
vsadds(pv,s)
VSTRING *pv;
char *s;
.s
vsfree(pv)
VSTRING *pv;
.s
.lm 8.rm 72.nhy .head mixed

.ps 58,80.lm 8.rm 72
.ax UTILITY LIBRARY INDEX
.t  ########C Utility Library
.st ########Utility Library Index
.lm 8.rm 72
.f
The following is a keyword in context index to the Utility Library.
The entry in the left-hand column is the title of the routine in
the main Library documentation.
.s 2
.nf
vstring         vstring                Add a character to a     
vstring         vstring                Add a string to a        
fxadd                                  Add an item to a flex    
fxaddo                                 Add ordered to a flex    
fxaddo             Locate last element added ordered to a flex  
fxitem          a flex     Obtain (the address of) an item from 
talloc          be easily deleted      Allocate space that can  
talloc                      Free space allocated by talloc()    
talloc                 Mark on current allocation thread        
isalpha              if a character is alphabetic           Test
isalfnum             if a character is alphanumeric         Test
match             leading substriDetermine if a string matches a
iovtoa            Convert file name to Ascii - obsolete         
talloc                  space that can be easily deletedAllocate
blkcmp                     Compare two blocks of memory         
blkeq           equality      Test two blocks of memory for     
time            ctime() andDefine time buffer structure for     
talloc             Allocate space that can be easily deleted    
chrtab          characteristics table  Character                
index                     Get index of character in string      
isdigit                      Test if a character is a digit     
isalpha                      Test if a character is alphabetic  
isalfnum        alphanumeric Test if a character is             
islower                      Test if a character is lowercase   
isupper                      Test if a character is uppercase   
iswhite                      Test if a character is whitespace  
chrtab             Header file for the character table chrtab[] 
strfns          string         Convert character to a new       
vstring                          Add a character to a vstring   
translate                           Do character translation    
replace                         Simple character translation    
translate       table            Clear character translation    
translate       table              Set character translation    
chrtab                       Character characteristics table    
esc                    Process escaped characters               
strfns          string   Take leftmost characters forming a new 
strfns          string  Take rightmost characters forming a new 
strfns          forming a new sExtract characters from a string 
chrtab             the character table chrtab[]  Header file for
translate       translation table      Clear character          
blkcmp          memory                 Compare two blocks of    
strfns          making new string      Concatenate strings      
strfns          new string             Convert character to a   
iovtoa          Ascii - obsolete       Convert file name to     
strfns                      Make a new copy of a string         
savestr                    Make a safe copy of a string         
flex                                   Create a new flex        
tables                                 Create a new table       
vstring                                Create a new vstring     
time              buffer structure for ctime() and rtDefine time
talloc          thread         Mark on current allocation       
time            structure for ctime() and rtime()e buffer       
tables                                 Delete a table element   
fxeject         flex            Eject (delete) an item from a   
talloc              that can be easily deleted    Allocate space
match           matches a leading substring of anotherstring    
isdigit            if a character is a digit                Test
translate                              Do character translation 
talloc               space that can be easily deleted   Allocate
fxeject         from a flex            Eject (delete) an item   
tables                  Delete a table element                  
tables                  Update a table element                  
fxaddo          a flex     Locate last element added ordered to 
tables                    Insert a new element into a table     
blkeq             blocks of memory for equality         Test two
esc                            Process escaped characters       
fxneed          shrink a flex          Explicitly grow or       
strfns          a string forming a new stringt characters from  
strfns          string forming a new stringact segment of a     
qsort                                  Fast in-core sort        
flex                            Header file for flexes          
chrtab          table chrtab[]  Header file for the character   
vstring         vstrings        Header file for using using     
iovtoa          obsolete       Convert file name to Ascii -     
fxadd                 Add an item to a flex                     
fxaddo                Add ordered to a flex                     
flex                      Create a new flex                     
fxeject                 an item from a flex       Eject (delete)
fxneed                grow or shrink a flex           Explicitly
fxinject         Inject an item into a flex                     
fxaddo              added ordered to a flex  Locate last element
fxitem              of) an item from a flex  Obtain (the address
fxset            value of an item in a flex              Set the
fxfree                   fxfree Free a flex                     
flex                   Header file for flexes                   
strfns                   from a string formingExtract characters
strfns             segment of a string forming a new striExtract
strfns             leftmost characters forming a new string Take
strfns            rightmost characters forming a new string Take
fxfree                          fxfree Free a flex              
vstring                                Free a vstring           
talloc          talloc()               Free space allocated by  
tables              Hash table handler functions                
fxfree                                 fxfree Free a flex       
instr           a string               Get index of a string in 
index           in string              Get index of character   
fxneed                      Explicitly grow or shrink a flex    
tables                      Hash table handler functions        
tables          functions              Hash table handler       
flex                                   Header file for flexes   
chrtab          character table chrtab[]eader file for the      
vstring         using vstrings         Header file for using    
isdigit         digit             Test if a character is a      
isalpha         alphabetic        Test if a character is        
isalfnum        alphanumeric      Test if a character is        
islower         lowercase         Test if a character is        
isupper         uppercase         Test if a character is        
iswhite         whitespace        Test if a character is        
match           leading substDetermine otherstring matches a    
qsort                             Fast in-core sort             
instr           string             Get index of a string in a   
index           string             Get index of character in    
fxinject        flex                   Inject an item into a    
tables          into a table           Insert a new element     
fxinject                Inject an item into a flex              
tables            Insert a new element into a table             
isdigit            Test if a character is a digit               
isalpha            Test if a character is alphabetic            
isalfnum           Test if a character is alphanumeric          
islower            Test if a character is lowercase             
isupper            Test if a character is uppercase             
iswhite            Test if a character is whitespace            
fxeject              Eject (delete) an item from a flex         
fxitem             (the address of) an item from a flex   Obtain
fxset              Set the value of an item in a flex           
fxinject                     Inject an item into a flex         
fxadd                           Add an item to a flex           
tables                          Look a key up in a table        
fxaddo          ordered to a fleLocate last element added       
match            if a string matches a leading substrinDetermine
strfns          forming a new striTake leftmost characters      
fxaddo          added ordered to a flexLocate last element      
tables                                 Look a key up in a table 
islower              if a character is lowercase            Test
strfns          string                 Make a new copy of a     
savestr         string                 Make a safe copy of a    
strfns             Concatenate strings making new string        
talloc          allocation thread      Mark on current          
match           sDetermine if a string matches a leading        
blkcmp           Compare two blocks of memory                   
blkeq               Test two blocks of memory for equality      
iovtoa                    Convert file name to Ascii - obsolete 
strfns                          Make a new copy of a string     
tables                        Insert a new element into a table 
flex                          Create a new flex                 
strfns                  strings making new string    Concatenate
strfns                  character to a new string        Convert
strfns              a string forming a neExtract characters from
strfns           of a string forming a new stringExtract segment
strfns            characters forming a new string  Take leftmost
strfns            characters forming a new string Take rightmost
tables                        Create a new table                
vstring                       Create a new vstring              
iovtoa            file name to Ascii - obsolete          Convert
fxitem          an item from a flex    Obtain (the address of)  
fxitem             Obtain (the address of) an item from a flex  
fxneed                 Explicitly grow or shrink a flex         
fxaddo                             Add ordered to a flex        
fxaddo              last element added ordered to a flex  Locate
esc             characters             Process escaped          
strfns          forming a new striTake rightmost characters     
time                   for ctime() aDefine time buffer structure
savestr                         Make a safe copy of a string    
tables                                 Scan through a table     
strfns          forming a new sExtract segment of a string      
translate       translation table      Set character            
fxset           in a flex              Set the value of an item 
fxneed              Explicitly grow or shrink a flex            
replace         translation            Simple character         
qsort                     Fast in-core sort                     
talloc          talloc()          Free space allocated by       
talloc          deleted       Allocate space that can be easily 
strfns              strings making new string        Concatenate
strfns              character to a new string            Convert
strfns            string forming a new Extract characters from a
strfns            string forming a new strinExtract segment of a
instr                 of a string in a string          Get index
index            index of character in string                Get
strfns            Make a new copy of a string                   
savestr          Make a safe copy of a string                   
strfns                   forming a new sTake leftmost characters
strfns                   forming a new Take rightmost characters
trim                 whitespace from a string      Trim trailing
strfns          stri characters from a string forming a nExtract
strfns          stExtract segment of a string forming a new     
instr                   Get index of a string in a string       
match           substrinDetermine if a string matches a leading 
vstring                          Add a string to a vstring      
strfns          string     Concatenate strings making new       
time            and Define time buffer structure for ctime()    
match                matches a leading subsDetermine if a string
chrtab                 characteristics table           Character
translate        character translation table               Clear
tables                    Create a new table                    
tables            a new element into a table              Insert
tables              Look a key up in a table                    
tables                  Scan through a table                    
translate        character translation table                 Set
chrtab               for the character table chrtab[]Header file
tables                        Delete a table element            
tables                        Update a table element            
tables                            Hash table handler functions  
strfns          forming a new string   Take leftmost characters 
strfns          characters forming a new stringhtmost           
talloc              space allocated by talloc()             Free
isdigit         digit                  Test if a character is a 
isalpha         alphabetic             Test if a character is   
isalfnum        alphanumeric           Test if a character is   
islower         lowercase              Test if a character is   
isupper         uppercase              Test if a character is   
iswhite         whitespace             Test if a character is   
blkeq           memory for equality    Test two blocks of       
talloc          deleted Allocate space that can be easily       
talloc           on current allocation thread               Mark
tables                            Scan through a table          
time            for ctime() and Define time buffer structure    
trim            a string          Trim trailing whitespace from 
translate                 Do character translation              
replace               Simple character translation              
translate              Clear character translation table        
translate                Set character translation table        
trim            from a string          Trim trailing whitespace 
blkcmp                         Compare two blocks of memory     
blkeq           equality          Test two blocks of memory for 
tables                      Look a key up in a table            
tables                                 Update a table element   
isupper              if a character is uppercase            Test
vstring                Header file for using using vstrings     
vstring          Header file for using using vstrings           
fxset           flex           Set the value of an item in a    
vstring           Add a character to a vstring                  
vstring              Add a string to a vstring                  
vstring                   Create a new vstring                  
vstring                         Free a vstring                  
vstring           file for using using vstrings           Header
iswhite              if a character is whitespace           Test
trim                     Trim trailing whitespace from a string 
